; Double-height six digit score
; by Lee Fastenau 03/24/2004

		processor 6502
		include "vcs.h"
		include "macro.h"
;		include "leelib.h"

; ------------------------------------
; Constant declarations
; ------------------------------------
frameDelay	equ 3
addScore	equ $0321	; Score is BCD, so use hex here ($0100 = 100pts, $0250 = 250pts)

		seg.u vars
variables	org $80
; ------------------------------------
; Variable declarations
; ------------------------------------
temp		byte		; Temporary storage
rowCount	byte		; Kernel row counter
frameCount	byte		; Frame counter (used with frameDelay to slow things down)
d0		word		; Digit pointers
d1		word
d2		word
d3		word
d4		word
d5		word
score		ds 3		; BCD score (1 nibble per digit)
;		REPORT_VAR_SPACE

		seg prog
program		org $F000
; ------------------------------------
; Program
; ------------------------------------
initAll subroutine
		CLEAN_START	; Clear mem/setup registers
		jsr initGame
		jsr gameLoop

; ------------------------------------
; Initialize game
; ------------------------------------
initGame subroutine
		lda #1
		sta VDELP0	; Set the delay registers for P0
		sta VDELP1	; And P1 ... Why?  Visit this link:
				; http://www.biglist.com/lists/stella/archives/199704/msg00137.html
		lda #$CE
		sta COLUP0	; Set the color registers for P0
		sta COLUP1	; And P1

		lda #3
		sta NUSIZ0	; Set the number size registers to show three clones for P0
		sta NUSIZ1	; And (you guessed it) P1

		lda #>imgData	; Set the MSB for each of our digit pointers
		sta d0+1	; They are all the same since they don't 
		sta d1+1	; cross any page boundaries
		sta d2+1
		sta d3+1
		sta d4+1
		sta d5+1
		
		lda #frameDelay
		sta frameCount	; Initialize frameCount
		rts
	
; ------------------------------------
; Game loop
; ------------------------------------
gameLoop subroutine
		VERTICAL_SYNC
		lda  #43	; Begin vblank timer
		sta  TIM64T

; ------------------------------------
; BEGIN vblank logic
; ------------------------------------

		dec frameCount	; Decrease frame counter
		bne .skipscore	; If not zero, skip score calculation
		lda #frameDelay	; Else
		sta frameCount	; Reset frame counter
		
.calcscore	sed		; Decimal mode
		clc		; Clear carry, we're getting ready to add
		lda score+2	; Load last 2 digits (BCD)
		adc #<addScore	; Add the LSB of addScore
		sta score+2
		lda score+1	; Load middle 2 digits
		adc #>addScore	; Add the MSB of addScore plus carry
		sta score+1
		lda score	; Load first 2 digits
		adc #$00	; Add carry
		sta score
		cld		; We're not in decimal mode anymore, Toto
.skipscore		


		ldy #5		; Set up score pointers (counts down 5 to 0)
.loop		tya		; 
		lsr		; Divide by 2 to get score byte offset (save carry)
		tax		; Move A to X for use as offset
		lda score,x	; Get 2 digits
		bcc .highnibble	; If carry clear, then use high nibble
		and #$0F	; else isolate low nibble
		asl		; Multiply by 8 (the height of our graphics)
		asl		; A will be used as the LSB in our pointer
		asl
		bcc .endnibble
.highnibble	and #$F0	; Isolate high nibble
		lsr		; Divide by two to get our pointer's LSB
.endnibble	sta temp	; Store LSB
		tya		; Calculate offset for the pointer itself
		asl		; Y*2 because each pointer is a word
		tax		; Move A to X for use as an offset
		lda temp	; Get our pointer's LSB
		sta d0,x	; And store it
		dey		; Decrease our counter
		bpl .loop	; Repeat for the next digit



		sta WSYNC	; Score positioning begins at hblank
				; Thus, the counting begins...
				;cycles	total	pixel position
		SLEEP 37	;37	37	43px	(43px = 37 cycles * 3 - 68 hblank)
		sta RESP0	;+3	40	52px	Since TIA moves at 3 color clocks [or pixels]
		sta RESP1	;+3	43	61px	per cycle, we have to multiply cycles by 3.
		lda #$10	; Nudge P0 1px left (P0 is at 51px, theoretically)
		sta HMP0
		lda #$20	; Nudge P1 2px left (p1 is at 59px, theoretically)
		sta HMP1
		sta WSYNC	; Sync again and
		sta HMOVE	; Move

; ------------------------------------
; END vblank logic
; ------------------------------------
	
.vblankWait	lda INTIM	; Wait for timer to expire
		bne .vblankWait
		sta WSYNC
		sta VBLANK 
		
		jmp scanLoop	; Easy to forget that "align" requires a preceeding jump
				; BEQ would work, but it would take 4 cycles
				; (2 cycles + 1 for branching + 1 for crossing page boundary)
		
		align 256
; ------------------------------------
; BEGIN scanline logic
; ------------------------------------
scanLoop subroutine
		sta WSYNC	; White line above score
		lda #$0E	; Set color
		sta COLUBK	; Store in register
		ldy #15		; This will be our row counter for the score
				; Counts down 15 to 0

		sta WSYNC	; And let's count again...
		lda #$02	;+2	2	Set background color for score
		sta COLUBK	;+3	3	Store in register
		SLEEP 57	;+57	62	The score loop doesn't use WSYNC,
				;		so we have to burn cycles to kind
				;		of "hit the ground running"
.score
				;cycles	total	pixel position
		sty rowCount	;+3	65	Y is multi-use, so we have to store it
		tya		;+2	70	Move Y to A
		lsr		;+2	72	and divide by 2
		tay		;+2	74	to give us double height Y offset
		lda (d0),y	;+5	76
				; This is where HBLANK begins!
				; Don't need WSYNC when exactly 76 cycles are used
		sta GRP0	;+3	3
		lda (d1),y	;+5	8
		sta GRP1	;+3	11	VDELP0/1 are integral to how this works
		lda (d2),y	;+5	16	See an EXCELLENT description of exactly
		sta GRP0	;+3	19	what's happening here:
		lda (d5),y	;+5	24
		sta temp	;+3	27	http://www.biglist.com/lists/stella/archives/199704/msg00137.html
		lda (d4),y	;+5	32
		tax		;+2	34
		lda (d3),y	;+5	39
		ldy temp	;+3	42	1st digit begins displaying here
		sta GRP1	;+3	45	(see score positioning above to see why)
		stx GRP0	;+3	48
		sty GRP1	;+3	51
		sta GRP0	;+3	54
		ldy rowCount	;+3	57
		dey		;+2	59
		bpl .score	;+3	62
		
		sta WSYNC	; Blank row
		lda #0		; Here, we reset the graphics registers
		sta GRP0
		sta GRP1
		sta GRP0
		sta GRP1
		
		sta WSYNC	; White line below score
		lda #$0E
		sta COLUBK
		sta WSYNC	; Black row
		lda #$00
		sta COLUBK
		
		ldy #192-21	; The above code uses 21 rows
.loop		sta WSYNC	; Fill the rest of the screen with ... 
		dey		; ... Nuthin'
		bne .loop
; ------------------------------------
; END scanline logic
; ------------------------------------

		lda #2
		sta WSYNC
		sta VBLANK
		ldx #30
.overscanLoop	sta WSYNC
		dex
		bne .overscanLoop
		jmp gameLoop

		align 256
; ------------------------------------
; Image data
; ------------------------------------
imgData
img0		hex 7C C6 C6 C6 C6 C6 C6 7C
img1		hex FE 18 18 18 18 78 38 18
img2		hex FE C0 60 3C 06 06 C6 7C
img3		hex 7C C6 06 06 3C 06 C6 7C
img4		hex 0C 0C FE CC CC CC 6C 0C
img5		hex 7C C6 06 06 FC C0 C0 FE
img6		hex 7C C6 C6 C6 FC C0 60 38
img7		hex 30 30 30 30 18 0C 06 FE
img8	 	hex 7C C6 C6 C6 7C C6 C6 7C
img9		hex 38 0C 06 7E C6 C6 C6 7C

		seg res
reset		org $FFFC
; ------------------------------------
; Reset vector
; ------------------------------------
		.word initAll
		.word initAll
	

